library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity cf is
    Port (
		OPB_Clk : in std_logic;
		OPB_Rst : in std_logic;
		opb_addr : in std_logic_vector( 31 downto 0 );
		cf_data : in std_logic_vector( 15 downto 0 );     -- data from pad_io (cf)
		cf_cs : in std_logic;
		cf_rdy : in std_logic;                            -- iordy signal from pad_io (cf)
		opb_data : inout std_logic_vector( 31 downto 0 );
		cf_addr : out std_logic_vector( 19 downto 0 );
		cf_ce0 : out std_logic;
		cf_ce1 : out std_logic;
		cf_oe : out std_logic;
		cf_busy : out std_logic;
		cf_xfer : out std_logic
	);
end cf;

architecture Behavioral of cf is

component cfctrl
Port ( rst : in std_logic;
	   clk : in std_logic;
	   cf_cs : in std_logic;       -- pushes state machine out of idle state
	   cf_rdy : in std_logic;      -- compactflash ready signal

	   --cf_go_addr : out std_logic;
	   cf_busy : out std_logic;    -- finished full read sequence
	   cf_ce : out std_logic;      -- cf chip enable
	   cf_oe : out std_logic;      -- cf output enable
	   cf_read : out std_logic;    -- data available as of this cycle
	   cf_bram_addr : out std_logic_vector( 7 downto 0 );
);
end component;

signal cf_rdata : std_logic_vector (15 downto 0);
signal cf_rdata0 : std_logic_vector (15 downto 0);
signal cf_rdata1 : std_logic_vector (15 downto 0);

signal cf_addr_stat : std_logic := '0';
signal cf_bram_counter : std_logic_vector( 7 downto 0 );
signal addr_mux : std_logic_vector(19 downto 0);
signal cs : std_logic;
signal addr : std_logic_vector (23 downto 0); 	
signal be : std_logic_vector (3 downto 0);
signal pb_bytesel : std_logic_vector (1 downto 0);

signal cf_addr_reg : std_logic_vector( 16 downto 0 );   -- block address to read off cf
signal xfer : std_logic := '0';                         -- let opb bus about xfer
--signal go_addr : std_logic := '0';                         -- let opb bus about xfer
signal busy : std_logic;
signal ce0 : std_logic;
signal ce1 : std_logic;
signal oe : std_logic;
signal write_enable : std_logic;
signal bram_select : std_logic := '0';
signal cf_read : std_logic;

begin

cf_controller : cfctrl port map (
	rst => OPB_Rst,
	clk => OPB_Clk,
	cf_cs => cs,
	cf_rdy => cf_rdy,
	--cf_go_addr => go_addr,
	cf_busy => busy,
	cf_ce0 => ce0,
	cf_ce1 => ce1,
	cf_oe => oe,
	cf_read => cf_read,
	cf_bram_addr => cf_bram_counter
);

cf_read0 : RAMB4_S16_S16 port map (
	-- write
	CLKA => OPB_Clk,
	RSTA => OPB_Rst,
	DIA => cf_data,
	ENA => '1',
	WEA => not bram_select and cf_read,
	ADDRA => cf_bram_counter,
	DOA => open,
	
	-- read
	CLKB => OPB_Clk,
	RSTB => OPB_Rst,
	DIB => X"0000",
	ENB => '1',
	WEB => '0',
	ADDRB => opb_addr,
	DOB => cf_rdata0
);

cf_read1 : RAMB4_S16_S16 port map (
	-- write
	CLKA => OPB_Clk,
	RSTA => OPB_Rst,
	DIA => cf_data,
	ENA => '1',
	WEA => bram_select and cf_read,
	ADDRA => cf_bram_counter,
	DOA => open,
	
	-- read
	CLKB => OPB_Clk,
	RSTB => OPB_Rst,
	DIB => X"0000",
	ENB => '1',
	WEB => '0',
	ADDRB => opb_addr,
	DOB => cf_rdata1
);

process( OPB_Clk, OPB_Rst )
begin
	if OPB_Rst = '1' then
		bram_select <= '0';
	elsif OPB_Clk'event and OPB_Clk = '1' then
		-- write to cf block addr register
		if opb_addr( 11 downto 0 ) = X"1FF" then
			if busy = '0' then
				cf_addr_reg <= opb_data( 16 downto 0 );
				xfer <= '1';
				cs <= '1';
				bram_select <= not bram_select;
			else
				xfer <= '0';
			end if;
		elsif opb_addr( 11 downto 8 ) = X"0" then
			cf_rdata_mux <= bram_select;

			if bram_select = '1' then
				cf_rdata <= cf_rdata0;
			else
				cf_rdata <= cf_rdata1;
			end if

			xfer <= '1';
		else
			xfer <= '0';
		end if;
	end if;
end process;

process( OPB_Clk, OPB_Rst )
begin
	if OPB_Rst = '1' then
	elsif OPB_Clk'event and OPB_Clk = '1' then
		opb_data <= X"0000" & cf_rdata;
	end if;
end process;

cf_addr( 19 downto 17 ) <= "101";
cf_addr( 16 downto 0 )  <= cf_addr_reg;
cf_ce0 <= ce0;
cf_ce1 <= ce1;
cf_xfer <= xfer;
--cf_go_addr <= go_addr;
cf_busy <= busy;
cf_oe <= oe;


end process;

end Behavioral;
